/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/std/serialize.hpp>
#include <ox/std/typetraits.hpp>

#include "alignmentcatcher.hpp"
#include "sizecatcher.hpp"

namespace ox {

struct NativePlatSpec {
	using PtrType = uintptr_t;
	using size_t = std::size_t;

	static constexpr auto PtrAlign = alignof(void*);
	static constexpr PtrType RomStart = 0x08000000;

	template<typename T>
	[[nodiscard]]
	static constexpr auto alignOf(const T &v) noexcept {
		if constexpr(ox::is_integral_v<T>) {
			return alignof(T);
		} else if constexpr(ox::is_pointer_v<T>) {
			return PtrAlign;
		} else {
			AlignmentCatcher<NativePlatSpec> c;
			oxAssert(model(c.interface(), &v), "Could not get alignment for type");
			return c.biggestAlignment;
		}
	}

	[[nodiscard]]
	static constexpr auto correctEndianness(auto v) noexcept {
		return v;
	}
};

template<typename PlatSpec, typename T>
[[nodiscard]]
constexpr std::size_t alignOf(const T &t) noexcept {
	return PlatSpec::alignOf(t);
}

template<typename PlatSpec, typename T>
constexpr auto alignOf_v = alignOf<PlatSpec>(static_cast<T*>(nullptr));

}
