/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/std/def.hpp>

#include <nostalgia/foundation/typeconv.hpp>

#include "context.hpp"
#include "gfx.hpp"

namespace nostalgia::core {

// Type converters

struct NostalgiaPaletteToPaletteConverter: public foundation::Converter<NostalgiaPalette, Palette> {
	ox::Error convert(foundation::Context*, NostalgiaPalette *src, Palette *dst) noexcept final {
		dst->colors = std::move(src->colors);
		return {};
	}
};

struct NostalgiaGraphicToTileSheetConverter: public foundation::Converter<NostalgiaGraphic, TileSheet> {
	ox::Error convert(foundation::Context*, NostalgiaGraphic *src, TileSheet *dst) noexcept final {
		dst->bpp              = src->bpp;
		dst->subsheet.name    = "Root";
		dst->subsheet.rows    = src->rows;
		dst->subsheet.columns = src->columns;
		dst->defaultPalette   = std::move(src->defaultPalette);
		dst->subsheet.pixels  = std::move(src->pixels);
		return {};
	}
};

struct TileSheetToCompactTileSheetConverter: public foundation::Converter<TileSheet, CompactTileSheet> {
	ox::Error convert(foundation::Context*, TileSheet *src, CompactTileSheet *dst) noexcept final {
		dst->bpp            = src->bpp;
		dst->defaultPalette = std::move(src->defaultPalette);
		dst->pixels         = src->pixels();
		return {};
	}
};

}
